/**
 * Example of how to define functions/handlers to be used in TCB services.
 * 
 * The services examples use the functions/handlers defined in this example.
 *
 * @author Paulo Sousa
 */

#include "stdafx.h"
#include "dll_example_for_tcb_handler.h"
#include "Winsock.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


/*TimelyExecution Service */
//function to be timely executed
extern "C" __declspec(dllexport)
void timely_example(void) {
	printf("TimelyExec Handler!\n");
}
//Worst Case Execution Time of timely_example (=1 millisecond)
extern "C" __declspec(dllexport)
int timely_example_wcet(void) {
	return 1;
}


/*LocalFailureDetection Service*/
//function to be executed if a local timing failure occurs
extern "C" __declspec(dllexport)
void localFD_example(void) {
	printf("LocalFailure Handler!\n");
}
//Worst Case Execution Time of localFD_example (=1 millisecond)
extern "C" __declspec(dllexport)
int localFD_example_wcet(void) {
	return 1;
}


/*DistributedFailureDetection Service*/
//function to be executed if a distributed timing failure occurs
extern "C" __declspec(dllexport)
void distFD_example(unsigned long from, int seq) {
	//from= tcb id of the msg sender
	//seq= distributed FD sequence number
	//create event to notify app of failure
	HANDLE failure_event = CreateEvent(NULL, TRUE, FALSE, L"failure_event");
	if (failure_event==NULL) {
		printf("Error creating failure event\n");
	}
	SetEvent(failure_event);
	CloseHandle(failure_event);
	//printf("DistFailure Handler! from=%u, seq=%d\n", from, seq);
}
//Worst Case Execution Time of distFD_example (=1 millisecond)
extern "C" __declspec(dllexport)
int distFD_example_wcet(void) {
	return 1;
}


/*Remote DistributedFailureDetection Service*/
//function to be executed if a remote distributed timing failure occurs
extern "C" __declspec(dllexport)
void remote_distFD_example(unsigned long from, int seq) {
	//from= tcb id of the msg sender
	//seq= distributed FD sequence number
	//create event to notify app of failure
	HANDLE failure_event = CreateEvent(NULL, TRUE, FALSE, L"remote_failure_event");
	if (failure_event==NULL) {
		printf("Error creating failure event\n");
	}
	SetEvent(failure_event);
	CloseHandle(failure_event);
	//printf("Remote DistFailure Handler! from=%u, seq=%d\n", from, seq);
}
//Worst Case Execution Time of remote_distFD_example (=1 millisecond)
extern "C" __declspec(dllexport)
int remote_distFD_example_wcet(void) {
	return 1;
}


/*QoS Monitoring Service*/
//function to be executed if a change occurs in QoS
/*IMPORTANT:
  The parameters mean, variance and num_values are only useful if you want to build
  a Probability Density Function. So, in this example, only the value parameter is
  "sent" to the application.
*/
extern "C" __declspec(dllexport)
void qos_handler_example(double value, double mean, double variance, int num_values) {
	//(coverage stability) value= new bound with specified coverage 
	//(coverage awareness) value= new coverage of specified bound

	
	
	//create memory mapping application<->DLL to send data to application
	HANDLE fmap = CreateFileMapping(INVALID_HANDLE_VALUE, 
		NULL, PAGE_READWRITE, 0, sizeof(double), 
		L"qos_memory_map");

	if (fmap==NULL) {
		printf("(QoS handler)Error in memory map\n");
		return;
	}
	
	//get pointer to memory map
	double *pt = (double *) 
			MapViewOfFile(fmap, FILE_MAP_WRITE, 0, 0, 0);
	
	if (pt==NULL) {
		printf("(QoS handler)Error in pointer to memory map\n");
		return;
	}

	//share new available data and inform application
	*pt = value;
	HANDLE qos_event = CreateEvent(NULL, TRUE, FALSE, L"qos_event");
	if (qos_event==NULL) {
		printf("Error creating QoS event\n");
	}
	SetEvent(qos_event);

	//close handles
	CloseHandle(fmap);
	CloseHandle(qos_event);

	//printf("QoS handler, new value: %.4f\n", value);
}
